// FILE: js/utilities/EngineService.js
var EngineService = function(restEngine, ZHPromise, eventAggregator, localStorageEngine, templateManager) {

    // ------------- Wahtch's ----------------------------------------------
    eventAggregator.subscribe('beforeProjectStartup', () => {
        checkLogetUser('beforeProjectStartup');
        templateManager.project.startSession = startSession;
    });

    eventAggregator.subscribe('onContainerOpen', function(called) {
        buildUrls();

        var container = called.container;
        var loginWindow = getLoginWindow();

        var containerNameLogin = loginWindow.split('#')[1];

        if (container.name !== containerNameLogin) {
            checkLogetUser('onContainerOpen');
        } else if (loginWindow === 'login#login') {
            var pathImage = getPathImage();
            container.widgets[0].fields[0].source = pathImage;
        }

    });

    // ----------------- Privates Functions ---------------------------------

    function buildUrls() {
        return null;
    }

    function checkLogetUser(mode) {
        var cookie = localStorage.getItem('login');
        var project = templateManager.project;
        if (!cookie || cookie === "notLogged") {
            eventAggregator.publish('notLogged');
            setLocalVar('login', 'notLogged');
            if (mode === 'beforeProjectStartup') {
                openWindow(getLoginWindow());
            } else if (mode === 'onContainerOpen') {
                openWindow(getLoginWindow());
            }
        } else {
            eventAggregator.publish('loginChecked');
        }
    }

    function getBhash() {
        var service = this.getLocalVar('bhash');
        if (service === undefined) {
            throw 'The bhash parameter, which had returned from authentication, disappeared from the cache memory.';
        } else {
            return service;
        }
    }

    // function getCookie(cname) {
    //     var name = cname + "=";
    //     var decodedCookie = decodeURIComponent(document.cookie);
    //     var ca = decodedCookie.split(';');
    //     for(var i = 0; i <ca.length; i++) {
    //         var c = ca[i];
    //         while (c.charAt(0) == ' ') {
    //             c = c.substring(1);
    //         }
    //         if (c.indexOf(name) === 0) {
    //             return c.substring(name.length, c.length);
    //         }
    //     }
    //     return "";
    // }

    function getLoginWindow() {
        var windowLogin = templateManager.project.loginWindow;
        if (!windowLogin) {
            return 'login#login';
        } else {
            return windowLogin;
        }
    }

    function getPathImage() {
        return templateManager.project.pathImage;
    }

    function getProductAlias() {
        var alias = templateManager.project.aliasBackProduct;
        if (alias === undefined) {
            throw 'Has no aliasBackProduct parameter.';
        } else {
            return alias;
        }
    }

    function getPhpsessionid() {
        var service = this.getLocalVar('tokenLogin');
        if (service === undefined) {
            throw 'The tokenLogin parameter, which had returned from authentication, disappeared from the cache memory.';
        } else {
            return service;
        }
    }

    function getUseService() {
        var service = templateManager.project.useService;
        if (service === undefined) {
            throw 'Has no useService parameter.';
        } else {
            return service;
        }
    }

    // function setCookie(cname, cvalue, exdays) {
    //     var expires;
    //     if (exdays) {
    //         var d = new Date();
    //         d.setTime(d.getTime() + (exdays*24*60*60*1000));
    //         expires = d.toUTCString();
    //     } else {
    //         expires = "0";
    //     }
    //     expires = "expires="+expires;
    //     document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/;domain=";
    // }

    function startSession(modules) {
        modules = modules || Object.keys(templateManager.project.aliasSubModules);
        var defer = ZHPromise.defer();
        var cookie = getLocalVar('login');
        if (cookie === "" || cookie === 'notLogged') {
            var crash1 = ZHPromise.defer();
            crash1.reject('Not logged.');
            return crash1;
        } else {
            var promises = [];
            var filter = [{
                "name": "USER",
                "operator": "=",
                "value": getLocalVar("USERDATA").USER
            }, {
                "name": "IS_LOGGED",
                "operator": "=",
                "value": cookie
            }, {
                "name": "LOGIN_TOKEN",
                "operator": "=",
                "value": getLocalVar("USERDATA").TOKEN
            }];

            modules.forEach(function(module) {
                promises.push(
                    restEngine.post(module + '#/lib_startSession', {
                        "requestType": "FilterData",
                        "filter": filter
                    })
                );
            });

            ZHPromise.all(promises).then(function(requestResult) {
                defer.resolve(requestResult);
            }, defer.reject);
        }

        return defer.promise;
    }

    this.saveAditionalData = (responseDataset) => {
        if (responseDataset.length > 0) {
            let userData = responseDataset[0];
            userData = userData.dataset.system;
            let CDFILIAL = userData.CDFILIAL;
            let NMFILIAL = userData.NMFILIAL;
            let NRORG = userData.NRORGTRAB;
            setLocalVar('CDFILIAL', CDFILIAL);
            setLocalVar('NMFILIAL', NMFILIAL);
            setLocalVar('NRORG', NRORG);
            window.NRORG = NRORG;
        }
    };

    this.setLoginByToken = (userData) => {
        return restEngine.post('sup#/setLoginByToken', {
            "requestType": "Row",
            "row": {
                nrorg: window.NRORG,
                email: userData.email,
                password: userData.password,
            }
        }).then(response => {
            if (response.dataset && response.dataset.token[0]) {
                let token = response.dataset.token[0];
                setLocalVar('authToken', token);
            } else {
                throw "Invalid response. No token provided";
            }
        });
    };

    // ----------------- Public Functions -----------------------------------
    this.authenticate = function(filter) {
        return new Promise((resolve, reject) => {
            restEngine.post('login#/login', {
                    "requestType": "FilterData",
                    "filter": filter
                })
                .then((requestResult) => {
                    this.setLocalVar('login', 'logged');
                    var userData = requestResult.dataset.userData;
                    return localStorageEngine.setDataSource('operator', userData).then(() => {
                        this.setLocalVar('tokenLogin', userData.TOKEN);
                        this.setLocalVar('bhash', userData.TOKEN);
                        this.setLocalVar('CDOPERADOR', userData.USER_ID);
                        this.setLocalVar('USER_NAME', userData.USER_NAME);
                        this.setLocalVar('USERDATA', userData);
                        return templateManager.project.startSession().then((userDataResponse) => {
                            eventAggregator.publish('loginSucess');
                            this.saveAditionalData(userDataResponse);
                            resolve(requestResult.dataset.userData);
                        });
                    });
                }).catch((e) => {
                    reject(e);
                });
        });

    };

    this.buildUrls = buildUrls;

    this.getProductAlias = getProductAlias;

    this.getLocalVar = getLocalVar;

    function getLocalVar(name) {
        return localStorageEngine.getLocalVar(name);
    }

    this.getUseService = getUseService;

    this.logout = function() {
        var defer = ZHPromise.defer();
        var modules = Object.keys(templateManager.project.aliasSubModules);
        var promises = [];
        modules.forEach(function(module) {
            promises.push(
                restEngine.post(module + '#/lib_logout', {
                    "requestType": "FilterData",
                    "filter": []
                })
            );
        });
        ZHPromise.all(promises).then(function() {
            localStorageEngine.setDataSource('operator', []);
            defer.resolve();
        }, defer.reject);

        return defer.promise;
    };

    this.setLocalVar = setLocalVar;

    function setLocalVar(name, baseUri) {
        localStorageEngine.setLocalVar(name, baseUri);
    }

    // this.setCookie = setCookie;

    this.testUrl = function(url) {
        return $.ajax({
            type: "POST",
            url: url + "/testeConnection",
            data: []
        });
    };

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    eventAggregator.subscribe("EventOnRequestStart", function(eventData) {
        var logoutRoute = 'lib_logout';
        var loginRoute = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        var route = requestConfig.url.split('/').pop();
        if (route != logoutRoute && route != loginRoute) {
            var token = getLocalVar('tokenLogin');
            var hash = localStorage.getItem('LOGIN_HASH');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if (window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if (eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if (hash) {
                requestConfig.headers['OAuth-Hash'] = hash;
                requestConfig.headers['OAuth-Project'] = window.tekLoginProjectId;
            }
            if (route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

};

Configuration(function(ContextRegister) {
    ContextRegister.register('EngineService', EngineService);
});


// FILE: js/utilities/UIManager.js
let UIManager = function(eventAggregator, templateManager) {

    /**
     * @type {Array<String|Function>}
     */
    let stateStack = [];

    let isListenerAttached = false;
    let activeWidget;
    let sf = this;
    /**
     * Called every time a container is open
     */
    this.init = () => {
        stateStack = [];
        activeWidget = this.activeWidget = templateManager.container.getActiveWidget();
    };

    if (!isListenerAttached) {
        eventAggregator.subscribe('onContainerOpen', this.init);
        isListenerAttached = true;
    }

    /**
     * Displays the given widget
     * @param {ZhWidgetOjbect} widgetObject
     * @param {boolean} [shouldReload] 
     */
    this.goToWidget = (widgetObject, shouldReload = false) => {
        if (widgetObject && widgetObject.name !== activeWidget.name) {
            widgetObject.activate();
            stateStack.push(activeWidget.name);
            activeWidget.isVisible = false;
            widgetObject.isVisible = true;
            activeWidget = widgetObject;
            if (activeWidget.container.steps) {
                templateManager.container.highlightStep(widgetToOpen.name);
                sf.moveSteps(widgetToOpen);
            }
            if (shouldReload) {
                return widgetObject.reload();
            }
        }
    };

    /**
     * Tryes to find a widget by it's name and display it
     * @param {string} widgetName 
     * @param {boolean} shouldReload 
     */
    this.goToWidgetByName = (widgetName, shouldReload = false, shouldCarryCurRow = false) => {
        let widgetToOpen = templateManager.container.getWidget(widgetName);
        if (widgetToOpen && widgetToOpen.name !== activeWidget.name) {
            widgetToOpen.activate();
            stateStack.push(activeWidget.name);
            activeWidget.isVisible = false;
            widgetToOpen.isVisible = true;
            if (shouldCarryCurRow) {
                widgetToOpen.currentRow = activeWidget.currentRow;
            }
            activeWidget = widgetToOpen;
            if (activeWidget.container.steps) {
                templateManager.container.highlightStep(widgetToOpen.name);
                sf.moveSteps(widgetToOpen);
            }

            if (shouldReload) {
                return widgetToOpen.reload();
            }
        } else {
            console.log('%can attempt to open the widget: ' + widgetName + " was made. But this widget wasn't found", "font-size: 15px");
        }
    };

    /**
     * Displays a widget that is already in the stack 
     * and removes all others in the path
     * 
     * homeScreen -> list -> listItem -> listItemDetails
     * 
     * goBackToStackedWidget('list')
     * 
     * homeScreen
     *  
     * @param {string} widgetName 
     */
    this.goBackToStackedWidget = widgetName => {
        let widgetToGoBack = templateManager.container.getWidget(widgetName);
        let widgetNameIndex = stateStack.indexOf(widgetName);
        if (widgetToGoBack && widgetNameIndex !== -1 && stateStack.length > 0) {
            stateStack = stateStack.slice(0, widgetNameIndex);
            widgetToGoBack.activate();
            activeWidget.isVisible = false;
            widgetToGoBack.isVisible = true;
            activeWidget = widgetToGoBack;
        }
    };

    /**
     * Displays the last widget in the stack
     * or triggers the last callback state in the stack 
     */
    this.goBack = () => {
        if (stateStack.length > 0) {
            if (typeof stateStack[stateStack.length - 1] === 'string') {
                let widgetToGoBackName = stateStack.pop();
                let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackName);
                widgetToGoBack.activate();
                widgetToGoBack.isVisible = true;
                activeWidget.isVisible = false;

                let steps = widgetToGoBack.container.steps;
                if (steps && steps.length > 0) {
                    let footerPositioner = document.getElementsByClassName("footer-positioner")[0];
                    let stepElements = document.getElementsByClassName("footer-wizard-step");

                    steps.find(step => step.active === true).active = false;
                    let newActiveStep = steps.find(step => step.name === widgetToGoBackName);
                    let indexOfStep = steps.indexOf(newActiveStep);
                    newActiveStep.active = true;

                    let curStep = stepElements[indexOfStep];
                    let curStepWidth = parseInt(window.getComputedStyle(curStep, null).getPropertyValue("width"));
                    footerPositioner.scrollLeft = curStep.offsetLeft - (curStepWidth / 3);
                }
                activeWidget = widgetToGoBack;
            } else if (typeof stateStack[stateStack.length - 1] === 'function') {
                (stateStack.pop())();
            }
        }
    };

    /**
     * hides the editor of the active widget, (if this widget is editable)
     */
    this.hideEdit = () => {
        if (activeWidget.hasOwnProperty('isEditing')) {
            this.allowScroll();
            activeWidget.isEditing = false;
            // activeWidget.restoreHeaderActions();
        }
    };

    this.preventScroll = () => document.body.classList.add('no-scroll');

    this.allowScroll = () => document.body.classList.remove('no-scroll');

    this.moveSteps = (widget) => {
        let steps = widget.container.steps;
        if (steps) {
            let footerPositioner = document.getElementsByClassName("footer-positioner")[0];
            let stepElements = document.getElementsByClassName("footer-wizard-step");

            let curStep = stepElements[steps.indexOf(steps.find(step => step.active === true))];
            let curStepWidth = parseInt(window.getComputedStyle(curStep, null).getPropertyValue("width"));
            footerPositioner.scrollLeft = curStep.offsetLeft - (curStepWidth / 3);
        }
    };

    /**
     * @param {Function} callback
     */
    this.pushState = (callback) => stateStack.push(callback);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('UIManager', UIManager);
});


// FILE: js/services/ApprovalService.js
var ApprovalService = function(restEngine) {

    this.approvesAlcada = function(currentRow, operacao, justificativa) {
        return restEngine.post(
            'sup#/aprovacaoAlcadas', {
                "requestType": "Row",
                "row": {
                    'OPERACAO': operacao,
                    'NRAUTOFORN': currentRow.NRDEMANDA,
                    'CDFILDEST': currentRow.CDFILDEST,
                    'CDFILLANC': currentRow.CDFILLANC,
                    'APROVADOR': currentRow.APROVADOR,
                    'CDOPERLOG': getLocalVar('CDOPERADOR'),
                    'NRCONFIGALCADA': currentRow.NRCONFIGALCADA,
                    'VRAPROVACAO': currentRow.VRAPROVCMP,
                    'NRAPROVALCADA': currentRow.NRAPROVALCADA,
                    'JUSTIFICATIVA': justificativa,
                    'CDFORNECED': currentRow.CDFORNECED,
                    'VRTOTALAF': currentRow.VRTOTALAF,
                    'IDTPALCADA': currentRow.IDTPALCADA,
                    'IDTPDEMANDA': currentRow.IDTPDEMANDA,
                    'IDTIPOVRAPROV': currentRow.IDTIPOVRAPROV,
                    'IDCONSISTVRAPROV': currentRow.IDCONSISTVRAPROV,
                    'CDFILCENTCMP': currentRow.CDFILCENTCMP,
                    'IDTIPOAF': currentRow.IDTIPOAF,
                    'IDLANCAFEXTRA': currentRow.IDLANCAFEXTRA,
                }
            }
        );
    };

    this.getConsisteOrca = function(currentRow) {
        return restEngine.post(
            'sup#/getConsisteOrca', {
                "requestType": "Row",
                "row": {
                    'NRAUTOFORN': currentRow.NRDEMANDA,
                    'CDFILDEST': currentRow.CDFILDEST,
                    'CDFILLANC': currentRow.CDFILLANC,
                }
            }
        );
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('ApprovalService', ApprovalService);
});


// FILE: js/services/LoginService.js
var LoginService = function(eventAggregator, EngineService, templateManager, ZHPromise, restEngine, MessageService, requestDataSourceEngine, localStorageEngine) {

    // ------------------------ Public Functions -------------------------

    var self = this;
    this.expiredSessionMethod = null;

    this.containsSericeUrls = function() {
        if (window.serviceUrls && window.serviceUrls.length > 0) {
            return true;
        } else {
            return false;
        }
    };

    this.getBaseUri = function() {
        return EngineService.getLocalVar('baseUri');
    };

    this.getProtocol = function() {
        return EngineService.getLocalVar('protocol');
    };

    this.login = function(data, attempts) {
        var filter;

        try {
            filter = [{
                name: "EMAIL",
                operator: "=",
                value: data.email
            }, {
                name: "PASSWORD",
                operator: "=",
                value: data.password
            }, {
                name: 'PRODUCT_ID',
                operator: '=',
                value: window.projectId
            }, {
                name: 'REQUESTER_URL',
                operator: '=',
                value: document.location.href
            }, {
                name: 'ATTEMPTS',
                operator: '=',
                value: attempts
            }];
        } catch (e) {
            var defer = ZHPromise.defer();
            defer.reject(e);
            return defer;
        }

        return EngineService.authenticate(filter);
    };

    this.logout = function() {
        return EngineService.logout().then(function() {
            EngineService.setLocalVar('login', 'notLogged');
            localStorage.removeItem('authToken');
            openWindow(templateManager.project.mainWindow);
        });
    };

    this.testUrl = function(baseUri) {
        try {
            alias = EngineService.getProductAlias();
        } catch (e) {
            var defer = ZHPromise.defer();
            defer.reject(e);
            return defer;
        }

        if (baseUri.indexOf(alias) === -1) {
            baseUri = baseUri + alias;
        }

        return EngineService.testUrl(baseUri).then(function() {
            EngineService.buildUrls();
            return baseUri;
        }, function() {
            return 'Address not found.';
        });

    };

    this.reLogin = () => {
        return new Promise((resolve, reject) => {
            let token = getLocalVar('authToken');
            let isLogged = (getLocalVar('login') === 'logged');
            if (token && isLogged) {
                return restEngine.post('sup#/getLoginByToken', {
                    "requestType": "Row",
                    "row": {
                        token: token
                    }
                }).then(userData => {
                    let dataToLogin = {
                        email: userData.dataset.user.EMAIL,
                        password: userData.dataset.user.PS,
                    };
                    this.login(dataToLogin, 1).then(response => {
                        templateManager.container.getActiveWidget().reload();
                        resolve();
                    }, error => {
                        reject(error);
                    });
                });
            } else {
                MessageService.getMessageByCode('NO_PROV_TOKEN').then(function(message) {
                    reject(message);
                });
            }
        });
    };

    eventAggregator.subscribe("EventOnRequestStart", function(eventData) {
        var validateExpiredSessionRoute = 'validateExpiredSessionApprove';
        var logoutRoute = 'lib_logout';
        var loginRoute = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        if (requestConfig && requestConfig.url && shouldAddRequestInfo(requestConfig.url) && templateManager.container) {
            requestConfig.data.origin = requestConfig.data.origin || {};
            var widget;
            var container = templateManager.loadingContainerJson;
            if (container) {
                widget = getFirstWidgetWithDataSourceRest(container.widgets);
                if (!widget) {
                    widget = container.widgets[0];
                }
            } else {
                container = templateManager.container;
                widget = container.getActiveWidget();
            }

            requestConfig.data.origin.containerLabel = container.label || container.name;
            requestConfig.data.origin.widgetLabel = widget.label || widget.name;
        }

        var route = requestConfig.url.split('/').pop();
        if (route != validateExpiredSessionRoute) {
            factoryExpiredSessionMethods();
        }
        if (route != logoutRoute && route != loginRoute) {
            var token = localStorage.getItem('LOGIN_TOKEN');
            var hash = localStorage.getItem('LOGIN_HASH');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if (window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if (eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if (hash) {
                requestConfig.headers['OAuth-Hash'] = hash;
                requestConfig.headers['OAuth-Project'] = window.tekLoginProjectId;
            }
            if (route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    var getFirstWidgetWithDataSourceRest = function(widgets) {
        for (var i = 0; i < widgets.length; i++) {
            if (widgets[i].dataSource && widgets[i].dataSource.rest == true) {
                return widgets[i];
            } else {
                return getFirstWidgetWithDataSourceRest(widgets[i]);
            }
        }
    };

    var factoryExpiredSessionMethods = function() {
        if (templateManager.project && templateManager.project.onRequestIdleTimeout) {
            if (self.expiredSessionMethod) {
                clearTimeout(self.expiredSessionMethod);
            }
            self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
        }
    };

    var validateExpiredSession = function() {
        var timeout = templateManager.project.onRequestIdleTimeout;
        var filter = [{
            name: 'TIMEOUT',
            value: timeout,
            operator: '='
        }];

        var userdata = getLocalVar('USERDATA');

        if (userdata && userdata.LOGGED) {
            requestDataSourceEngine.getDataSource('sup#/validateExpiredSessionApprove', filter).then(function(response) {
                if (response && response.dataset && response.dataset.expiredSessionInfo) {
                    var expiredSessionInfo = response.dataset.expiredSessionInfo;
                    if (expiredSessionInfo.IS_EXPIRED) {
                        eventAggregator.publish('LibExpiredSession');
                    } else {
                        if (self.expiredSessionMethod) {
                            clearTimeout(self.expiredSessionMethod);
                        }
                        Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
                    }
                }
            }).catch(() => {
                expiredLogout();
            });
        }
    };

    function logoutExpiredSession() {
        return EngineService.logout().then(function() {
            EngineService.setLocalVar('login', 'notLogged');
            localStorage.removeItem('authToken');
            openWindow(templateManager.project.mainWindow);

            var userData = getLocalVar('USERDATA');

            var newUserData = {
                'USER_ID': userData.USER_ID,
                'USER_NAME': userData.USER_NAME,
                'USER_EMAIL': userData.USER_EMAIL,
                'TOKEN': null,
                'LOGGED': false,
            };

            localStorageEngine.setLocalVar('USERDATA', newUserData);
        });
    }

    function expiredLogout() {
        if ('device' in window) { // if isn't running in device do nothing
            let deviceDataToDelete = {
                UUID: device.uuid,
                NRAPLICATIVO: 5
            };
            restEngine.post('sup#/crud/device/remove', {
                "requestType": "Row",
                "row": deviceDataToDelete
            }).then(logoutExpiredSession());
        } else {
            logoutExpiredSession();
        }
    }

    eventAggregator.subscribe("LibExpiredSession", function() {
        if (!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            expiredLogout();
        }
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('LoginService', LoginService);
});


// FILE: js/controllers/ApprovalController.js
var ApprovalController = function(ApprovalService) {};

Configuration(function(ContextRegister) {
    ContextRegister.register('ApprovalController', ApprovalController);
});


// FILE: js/controllers/LoginController.js
var LoginController = function(EngineService, LoginService, ScreenService, templateManager, UIManager, restEngine, MessageService, ZHPromise) {

    const TYPE_EXPIRED_PASSWORD = "EXPIRED_PASSWORD";

    var setLoginAttempts = function() {
        var loginWidget = templateManager.container.getWidget("login");
        loginWidget.attempts = 1;
    };

    var afterLoginSuccess = function(userData) {
        ScreenService.hideLoader();
        if (userData.LOGGED) {
            if (userData.EXPIRED_PASSWORD) {
                expiredPasswordValidation(userData);
            }
            ScreenService.openWindow('tasks');
        }
    };

    var expiredPasswordValidation = function(userData) {
        var message = 'MSG_RESET_PASSWORD';
        if (userData.EXPIRED_PASSWORD == TYPE_EXPIRED_PASSWORD) {
            message = 'MSG_EXPIRED_PASSWORD';
        }
        MessageService.showMessageByCode(message);
    };

    var afterLoginError = function(error) {
        var loginWidget = templateManager.container.getWidget("login");
        ScreenService.hideLoader();
        if (error && error.errorCode) {
            /** Error code 29: If miss the password again, the user will be blocked */
            /** Error code 31: Access blocked by attempts */
            /** Error code 32: Invalid user and password */
            var errorCode = error.errorCode;
            if (errorCode == 29 || errorCode == 32) {
                loginWidget.attempts++;
            } else if (errorCode == 31) {
                loginWidget.attempts = 1;
            } else if (errorCode === 5 || errorCode === 32) {
                MessageService.showMessageByCode('USR_PASS_INCOR');
            } else if (errorCode === 2) {
                MessageService.showMessageByCode('USR_INAC');
            }
        }
    };

    this.onEnter = function(widget) {
        this.setVisibilityForValidQRCode(widget);
        this.setUpDNS();
        setLoginAttempts();
        LoginService.reLogin().then(() => { // one piece of the code bellow is repeated because Safari doesn't suports Promise.finally
            if ('FirebasePlugin' in window) {
                window.FirebasePlugin.confirmListenerAttached();
            }
            this.updateFCMToken();
        }).catch(() => {
            if ('FirebasePlugin' in window) {
                window.FirebasePlugin.confirmListenerAttached();
            }
        });
        widget.currentRow.email = getLocalVar("USERDATA") ? getLocalVar("USERDATA").EMAIL : '';
        if (getLocalVar('login') === 'logged' && getLocalVar('tokenLogin') && getLocalVar('CDOPERADOR')) {
            ScreenService.openWindow('tasks');
        } else {
            setLocalVar('login', 'notLogged');
            setLocalVar('tokenLogin', null);
            setLocalVar('CDOPERADOR', null);
        }
    };

    this.login = function(widget) {
        var baseUri = window.serviceUrl;
        var attempts = widget.attempts;
        ScreenService.showLoader();
        LoginService.testUrl(baseUri).then(() => {
            if (LoginService.containsSericeUrls()) {
                if (widget.isValid()) {
                    LoginService.login(widget.currentRow, attempts).then((response) => {
                        afterLoginSuccess(response);
                        EngineService.setLoginByToken(widget.currentRow);
                        this.updateFCMToken(true);
                    }, function(error) {
                        afterLoginError(error);
                    });
                }
                document.getElementsByClassName('zh-background-loading')[0].style.display = 'none';
            } else {
                ScreenService.openPopup(templateManager.container.getWidget('settings'));
            }
        }, function(error) {
            if (error == 'Address not found.') {
                MessageService.showMessageByCode('ADDR_NOT_FOUND');
            }
            document.getElementsByClassName('zh-background-loading')[0].style.display = 'none';
        });
    };

    this.setBaseUri = function(widget) {
        var baseUri = widget.currentRow.baseuri;
        ScreenService.showLoader();
        LoginService.testUrl(baseUri).then(function() {
            ScreenService.hideLoader();
            ScreenService.closePopup();
            if (!templateManager.project.notTry) {
                this.login(templateManager.container.getWidget('login'));
            }
        }.bind(this), function(error) {
            ScreenService.hideLoader();
            ScreenService.showMessage(error);
        });
    };

    this.setValueField = function(widget) {
        var baseUri = LoginService.getBaseUri();
        if (baseUri) {
            widget.currentRow.baseuri = baseUri;
        }
    };

    this.logout = () => {
        if ('device' in window) { // if isn't running in device do nothing
            let deviceDataToDelete = {
                UUID: device.uuid,
                NRAPLICATIVO: 5
            };
            restEngine.post('sup#/crud/device/remove', {
                "requestType": "Row",
                "row": deviceDataToDelete
            }).then(LoginService.logout);
        } else {
            LoginService.logout();
        }
    };

    this.saveQRCodeData = (qrCodeData) => {
        for (const qrCodePropertyKey in qrCodeData) {
            const qrCodeProperty = qrCodeData[qrCodePropertyKey];
            localStorage.setItem(qrCodePropertyKey, qrCodeProperty);
        }
    };

    // TODO
    this.validateQRCode = (qrCodeValue) => {
        let splitedData = qrCodeValue.split(';');
        if (!splitedData || splitedData.length !== 2) {
            MessageService.getMessageByCode('INVAL_QRCODE').then(function(message) {
                return Promise.reject(message);
            });
        }
        let qrCodeData = {
            serviceURL: splitedData[0],
            NRORG: splitedData[1]
        };

        // TODO
        // do a request to validate QRCode
        qrCodeData.isQRCodeValid = true;

        return Promise.resolve(qrCodeData);
    };

    this.showQRCodeScanner = () => {
        let qrCodeWidget = templateManager.container.getWidget('qrCodeScanner');
        UIManager.goToWidgetByName('qrCodeScanner');
        this.readQRCode(qrCodeWidget);
        qrCodeWidget.__showCameraPreview();
    };

    this.readQRCode = function(qrCodeWidget) {
        qrCodeWidget.startScan(false).then((qrCodeValue) => {
            this.validateQRCode(qrCodeValue).then((qrCodeData) => {
                this.saveQRCodeData(qrCodeData);
                this.setVisibilityForValidQRCode();
                this.setUpDNS();
                UIManager.goBack(); // closes QRCode camera
            }).catch(() => {
                MessageService.showMessageByCode('INVAL_QRCODE');
                this.readQRCode(qrCodeWidget);
            });
        });
    };

    this.setVisibilityForValidQRCode = (widget) => {
        widget = widget || templateManager.container.getWidget('login');
        let validQRCode = localStorage.getItem('isQRCodeValid');
        if (validQRCode || widget.shouldIgnoreQrcode === true) {
            widget.fields.forEach(field => field.isVisible = true);
            let loginAction = widget.actions.find(action => action.name === 'login');
            loginAction.isVisible = true;
        }
    };

    this.setUpDNS = () => {
        let newURL = localStorage.getItem('serviceURL');
        if (newURL) {
            let oldServiceUrl = window.serviceUrl;
            window.serviceUrl = newURL;
            Object.keys(window.serviceUrls).forEach(function(serviceUrlKey) {
                let newBaseUrl = window.serviceUrls[serviceUrlKey].baseUrl;
                newBaseUrl = oldServiceUrl ? newBaseUrl.replace(oldServiceUrl, newURL) : baseUrl.replace(`<>BASE_URI<>`, newURL);
                window.serviceUrls[serviceUrlKey].baseUrl = newBaseUrl;
            });
            templateManager.updateURL(newURL);
        }
    };

    this.onManualQRCodeTyped = (typedValue) => {
        this.validateQRCode(typedValue).then((qrCodeData) => {
            this.saveQRCodeData(qrCodeData);
            this.setVisibilityForValidQRCode();
            this.setUpDNS();
            UIManager.goBack(); // closes QRCode camera
        }).catch((err) => {
            MessageService.showMessageByCode('INVAL_QRCODE');
        });
    };

    let sendFCMTokenSaveRequest = (token, __is_new = false) => {
        return restEngine.post('sup#/crud/device/save', {
            "requestType": "Row",
            "row": {
                UUID: device.uuid,
                PLATFORM: device.platform,
                TOKEN: token,
                CDOPERADOR: getLocalVar('CDOPERADOR'),
                NRORG: localStorage.getItem('NRORG'),
                __is_new: __is_new,
                NRAPLICATIVO: 5,
            }
        }).then(() => {
            delete window.shouldUpdateFCMToken;
        });
    };

    this.updateFCMToken = (forceUpdate = false) => {
        let token = localStorage.getItem('fcmToken');
        if (token && (window.shouldUpdateFCMToken || forceUpdate)) {
            return sendFCMTokenSaveRequest(token).catch(() => sendFCMTokenSaveRequest(token, true));
        } else {
            ZHPromise.reject();
        }
    };

    /**
     * checks if the user is not logged for preventing the android 
     * back button from going back to the account container after a logout
     */
    this.loginGuard = () => {
        if (getLocalVar('login') !== 'logged' || !getLocalVar('tokenLogin') || !getLocalVar('CDOPERADOR')) {
            ScreenService.openWindow('login');
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('LoginController', LoginController);
});


// FILE: js/controllers/TasksController.js
var TasksController = function(LoginService, ScreenService, templateManager, ZHPromise) {

    this.changeFilter = function(newData) {
        var widget = templateManager.container.getActiveWidget();
        let shouldOpenTasksContainer = templateManager.container.name !== newData.containerName;
        if (shouldOpenTasksContainer) {
            ScreenService.openWindow('tasks');
        }
        // setTimeout(function() {
        let loadData = () => {
            widget = templateManager.container.getActiveWidget();
            var filter = widget.dataSourceFilter;
            var filterElement = filter.find(function(filterEl) {
                return filterEl.name === 'IDAPROVADO';
            });
            filterElement.value = newData.filterBy;
            widget.title = newData.name;
            let CDOPERADORFilter = widget.dataSourceFilter.find(filter => filter.name === 'CDOPERADOR');
            if (newData.filterBy === 'PENDENTE') {
                widget.fields.forEach(function(action) {
                    action.isVisible = true;
                });
                CDOPERADORFilter.value = getLocalVar('CDOPERADOR');
                widget.filterValues.OVERVIEW = 'overviewInactive';
                let OVERVIEWFilterProperties = widget.filterProperties.fields.find(field => field.name === 'OVERVIEW');
                OVERVIEWFilterProperties.disabled = true;
            } else {
                widget.fields.forEach(function(action) {
                    action.isVisible = false;
                });
                let OVERVIEWFilterProperties = widget.filterProperties.fields.find(field => field.name === 'OVERVIEW');
                OVERVIEWFilterProperties.disabled = false;
                // CDOPERADORFilter.value = "T";
            }
            scroll(0, 0);
            widget.reload();
        };
        if (shouldOpenTasksContainer) {
            setTimeout(loadData, 100);
        } else {
            loadData();
        }
        // }, 100);
    };

    this.openDetailTab = function(widget) {
        widget.currentRow = widget.parent.parent.currentRow;

        let zhLoader = document.querySelector('div[data-zh-loader]');

        let loadPromises = [];
        let timelineWidget = widget.container.getWidget('Timeline');
        let itensWidget = widget.container.getWidget('itens');

        loadPromises.push(this.openTimeline(timelineWidget));
        loadPromises.push(this.openItens(itensWidget));

        ZHPromise.all(loadPromises).finally(function() {
            setTimeout(function() {
                zhLoader.style.display = 'none';
                zhLoader.classList.remove('force-display-none');
            }, 300);
        });
    };
    this.loadedChartDataSource = function(widget) {
        let field = widget.getField('approvalChart');

        let itens = field.itens;
        let total = itens.find(function(filterEl) {
            return filterEl.isTotal === true;
        });
        let start = itens.find(function(filterEl) {
            return filterEl.isStartValue === true;
        });
        let curAdition = itens.find(function(filterEl) {
            return filterEl.isCurrAdition === true;
        });

        // start.valor = widget.currentRow[start.id] ? widget.currentRow[start.id] : widget.dataSource.data[0][start.id];
        // total.valor = widget.currentRow[total.id] ? widget.currentRow[total.id] : widget.dataSource.data[0][total.id];
        start.valor = widget.dataSource.data[0][start.id];
        curAdition.valor = widget.dataSource.data[0].VRTOTALAF;
        total.valor = widget.dataSource.data[0][total.id];

        if (start.valor >= total.valor) {
            field.itens[0].color = "#DB0000";
        } else {
            field.itens[0].color = "#0039A6";
        }

        if (curAdition.valor >= total.valor) {
            field.itens[1].color = "#DB0000";
        } else {
            field.itens[1].color = "#56CCF2";
        }

        start.position = 2;
        curAdition.position = 1;
        total.position = 0;

        start.percentage = getPercentage(start, total);
        curAdition.percentage = getPercentage(curAdition, total);
        total.percentage = getPercentage(total, total);

        if (start.percentage == 180) {
            curAdition.percentage = 0;
        }

        let chartBars = document.getElementsByClassName('bar');
        let animationArr = [];

        setTimeout(function() {
            for (let i = 0; i < chartBars.length; i++) {
                let animationValue = parseInt(chartBars[i].style.transform.replace('rotate(', '').replace('deg)', ''));
                chartBars[i].style.transform = 'rotate(' + (0) + 'deg)';
                animationArr.push(animationValue);
            }
            for (let i = 0; i < chartBars.length; i++) {
                setTimeout(function() { // jshint ignore:line
                    chartBars[i].classList.add('animation-start');
                    chartBars[i].style.transform = 'rotate(' + animationArr[i] + 'deg)';
                }, 600);
            }
        }, 10);
    };

    function getPercentage(actualField, totalField) {
        const totalPercent = 180;
        const initPercent = 0;
        if (actualField.isTotal) {
            return 180;
        }
        const newPercent = (actualField.valor * totalPercent) / totalField.valor;
        if (newPercent > totalPercent) {
            return totalPercent;
        }
        if (newPercent <= initPercent) {
            return initPercent;
        }
        return newPercent;
    }

    this.openTimeline = function(widget) {
        widget.currentRow = widget.parent.parent.currentRow;
        var filter = [{
            "name": "NRCONFIGALCADA",
            "operator": "=",
            "value": widget.currentRow.NRCONFIGALCADA
        }, {
            "name": "CDFILDEST",
            "operator": "=",
            "value": widget.currentRow.CDFILDEST
        }, {
            "name": "NRDEMANDA",
            "operator": "=",
            "value": widget.currentRow.NRDEMANDA
        }, {
            "name": "CDFILLANC",
            "operator": "=",
            "value": widget.currentRow.CDFILLANC
        }];
        widget.dataSourceFilter = filter;
        return widget.reload();
    };

    this.openItens = function(widget) {
        widget.currentRow = widget.parent.parent.currentRow;
        var filter = [{
            "name": "CDFILDEST",
            "operator": "=",
            "value": widget.currentRow.CDFILDEST
        }, {
            "name": "CDFILLANC",
            "operator": "=",
            "value": widget.currentRow.CDFILLANC
        }, {
            "name": "NRAUTOFORN",
            "operator": "=",
            "value": widget.currentRow.NRDEMANDA
        }];
        widget.dataSourceFilter = filter;
        return widget.reload();
    };

    function translateIntervalDate(date) {
        const dateFormat = 'DD/MM/YYYY';
        let baseDate;
        let modality;
        switch (date) {
            case 'hoje':
                baseDate = moment();
                modality = 'day';
                break;
            case 'amanha':
                baseDate = moment().add(1, 'day');
                modality = 'day';
                break;
            case 'ontem':
                baseDate = moment().subtract(1, 'day');
                modality = 'day';
                break;
            case 'proxima_semana':
                baseDate = moment().add(1, 'weeks');
                modality = 'week';
                break;
            case 'semana_anterior':
                baseDate = moment().subtract(1, 'weeks');
                modality = 'week';
                break;
            case 'semana_atual':
                baseDate = moment();
                modality = 'week';
                break;
            case 'proximo_mes':
                baseDate = moment().add(1, 'months');
                modality = 'month';
                break;
            case 'mes_anterior':
                baseDate = moment().subtract(1, 'months');
                modality = 'month';
                break;
            case 'mes_atual':
                baseDate = moment();
                modality = 'month';
                break;
        }
        return {
            start: baseDate.startOf(modality).format(dateFormat),
            end: baseDate.endOf(modality).format(dateFormat),
        };
    }

    this.setDateFilter = function(filterValues, widget) {
        let dataInicialFilter = widget.dataSourceFilter.find(filter => filter.name === 'DTINICIAL');
        let dataFinalFilter = widget.dataSourceFilter.find(filter => filter.name === 'DTFINAL');

        const {
            start,
            end
        } = translateIntervalDate(filterValues.DATA);

        dataInicialFilter.value = start;
        dataFinalFilter.value = end;
    };

    this.setOverviewFilter = function(filterValues, widget) {
        let CDOPERADORFilter = widget.dataSourceFilter.find(filter => filter.name === 'CDOPERADOR');
        let overviewStatus = filterValues.OVERVIEW;

        widget = templateManager.container.getActiveWidget();
        var filter = widget.dataSourceFilter;
        var filterElement = filter.find(function(filterEl) {
            return filterEl.name === 'IDAPROVADO';
        });

        if (overviewStatus === 'overviewActive' && filterElement.value !== 'PENDENTE') {
            CDOPERADORFilter.value = "T";
        } else if (overviewStatus === 'overviewInactive') {
            CDOPERADORFilter.value = getLocalVar('CDOPERADOR');
        }
    };

    this.prepareDataSourceFilter = function(filterValues) {
        const widget = templateManager.container.getWidget('alcadas');
        this.setDateFilter(filterValues, widget);
        this.setOverviewFilter(filterValues, widget);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('TasksController', TasksController);
});

