var EngineService = function(restEngine, ZHPromise, eventAggregator, localStorageEngine, templateManager) {

    // ------------- Wahtch's ----------------------------------------------
    eventAggregator.subscribe('beforeProjectStartup', () => {
        checkLogetUser('beforeProjectStartup');
        templateManager.project.startSession = startSession;
    });

    eventAggregator.subscribe('onContainerOpen', function(called) {
        buildUrls();

        var container = called.container;
        var loginWindow = getLoginWindow();

        var containerNameLogin = loginWindow.split('#')[1];

        if (container.name !== containerNameLogin) {
            checkLogetUser('onContainerOpen');
        } else if (loginWindow === 'login#login') {
            var pathImage = getPathImage();
            container.widgets[0].fields[0].source = pathImage;
        }

    });

    // ----------------- Privates Functions ---------------------------------

    function buildUrls() {
        return null;
    }

    function checkLogetUser(mode) {
        var cookie = localStorage.getItem('login');
        var project = templateManager.project;
        if (!cookie || cookie === "notLogged") {
            eventAggregator.publish('notLogged');
            setLocalVar('login', 'notLogged');
            if (mode === 'beforeProjectStartup') {
                openWindow(getLoginWindow());
            } else if (mode === 'onContainerOpen') {
                openWindow(getLoginWindow());
            }
        } else {
            eventAggregator.publish('loginChecked');
        }
    }

    function getBhash() {
        var service = this.getLocalVar('bhash');
        if (service === undefined) {
            throw 'The bhash parameter, which had returned from authentication, disappeared from the cache memory.';
        } else {
            return service;
        }
    }

    // function getCookie(cname) {
    //     var name = cname + "=";
    //     var decodedCookie = decodeURIComponent(document.cookie);
    //     var ca = decodedCookie.split(';');
    //     for(var i = 0; i <ca.length; i++) {
    //         var c = ca[i];
    //         while (c.charAt(0) == ' ') {
    //             c = c.substring(1);
    //         }
    //         if (c.indexOf(name) === 0) {
    //             return c.substring(name.length, c.length);
    //         }
    //     }
    //     return "";
    // }

    function getLoginWindow() {
        var windowLogin = templateManager.project.loginWindow;
        if (!windowLogin) {
            return 'login#login';
        } else {
            return windowLogin;
        }
    }

    function getPathImage() {
        return templateManager.project.pathImage;
    }

    function getProductAlias() {
        var alias = templateManager.project.aliasBackProduct;
        if (alias === undefined) {
            throw 'Has no aliasBackProduct parameter.';
        } else {
            return alias;
        }
    }

    function getPhpsessionid() {
        var service = this.getLocalVar('tokenLogin');
        if (service === undefined) {
            throw 'The tokenLogin parameter, which had returned from authentication, disappeared from the cache memory.';
        } else {
            return service;
        }
    }

    function getUseService() {
        var service = templateManager.project.useService;
        if (service === undefined) {
            throw 'Has no useService parameter.';
        } else {
            return service;
        }
    }

    // function setCookie(cname, cvalue, exdays) {
    //     var expires;
    //     if (exdays) {
    //         var d = new Date();
    //         d.setTime(d.getTime() + (exdays*24*60*60*1000));
    //         expires = d.toUTCString();
    //     } else {
    //         expires = "0";
    //     }
    //     expires = "expires="+expires;
    //     document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/;domain=";
    // }

    function startSession(modules) {
        modules = modules || Object.keys(templateManager.project.aliasSubModules);
        var defer = ZHPromise.defer();
        var cookie = getLocalVar('login');
        if (cookie === "" || cookie === 'notLogged') {
            var crash1 = ZHPromise.defer();
            crash1.reject('Not logged.');
            return crash1;
        } else {
            var promises = [];
            var filter = [{
                "name": "USER",
                "operator": "=",
                "value": getLocalVar("USERDATA").USER
            }, {
                "name": "IS_LOGGED",
                "operator": "=",
                "value": cookie
            }, {
                "name": "LOGIN_TOKEN",
                "operator": "=",
                "value": getLocalVar("USERDATA").TOKEN
            }];

            modules.forEach(function(module) {
                promises.push(
                    restEngine.post(module + '#/lib_startSession', {
                        "requestType": "FilterData",
                        "filter": filter
                    })
                );
            });

            ZHPromise.all(promises).then(function(requestResult) {
                defer.resolve(requestResult);
            }, defer.reject);
        }

        return defer.promise;
    }

    this.saveAditionalData = (responseDataset) => {
        if (responseDataset.length > 0) {
            let userData = responseDataset[0];
            userData = userData.dataset.system;
            let CDFILIAL = userData.CDFILIAL;
            let NMFILIAL = userData.NMFILIAL;
            let NRORG = userData.NRORGTRAB;
            setLocalVar('CDFILIAL', CDFILIAL);
            setLocalVar('NMFILIAL', NMFILIAL);
            setLocalVar('NRORG', NRORG);
            window.NRORG = NRORG;
        }
    };

    this.setLoginByToken = (userData) => {
        return restEngine.post('sup#/setLoginByToken', {
            "requestType": "Row",
            "row": {
                nrorg: window.NRORG,
                email: userData.email,
                password: userData.password,
            }
        }).then(response => {
            if (response.dataset && response.dataset.token[0]) {
                let token = response.dataset.token[0];
                setLocalVar('authToken', token);
            } else {
                throw "Invalid response. No token provided";
            }
        });
    };

    // ----------------- Public Functions -----------------------------------
    this.authenticate = function(filter) {
        return new Promise((resolve, reject) => {
            restEngine.post('login#/login', {
                    "requestType": "FilterData",
                    "filter": filter
                })
                .then((requestResult) => {
                    this.setLocalVar('login', 'logged');
                    var userData = requestResult.dataset.userData;
                    return localStorageEngine.setDataSource('operator', userData).then(() => {
                        this.setLocalVar('tokenLogin', userData.TOKEN);
                        this.setLocalVar('bhash', userData.TOKEN);
                        this.setLocalVar('CDOPERADOR', userData.USER_ID);
                        this.setLocalVar('USER_NAME', userData.USER_NAME);
                        this.setLocalVar('USERDATA', userData);
                        return templateManager.project.startSession().then((userDataResponse) => {
                            eventAggregator.publish('loginSucess');
                            this.saveAditionalData(userDataResponse);
                            resolve(requestResult.dataset.userData);
                        });
                    });
                }).catch((e) => {
                    reject(e);
                });
        });

    };

    this.buildUrls = buildUrls;

    this.getProductAlias = getProductAlias;

    this.getLocalVar = getLocalVar;

    function getLocalVar(name) {
        return localStorageEngine.getLocalVar(name);
    }

    this.getUseService = getUseService;

    this.logout = function() {
        var defer = ZHPromise.defer();
        var modules = Object.keys(templateManager.project.aliasSubModules);
        var promises = [];
        modules.forEach(function(module) {
            promises.push(
                restEngine.post(module + '#/lib_logout', {
                    "requestType": "FilterData",
                    "filter": []
                })
            );
        });
        ZHPromise.all(promises).then(function() {
            localStorageEngine.setDataSource('operator', []);
            defer.resolve();
        }, defer.reject);

        return defer.promise;
    };

    this.setLocalVar = setLocalVar;

    function setLocalVar(name, baseUri) {
        localStorageEngine.setLocalVar(name, baseUri);
    }

    // this.setCookie = setCookie;

    this.testUrl = function(url) {
        return $.ajax({
            type: "POST",
            url: url + "/testeConnection",
            data: []
        });
    };

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    eventAggregator.subscribe("EventOnRequestStart", function(eventData) {
        var logoutRoute = 'lib_logout';
        var loginRoute = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        var route = requestConfig.url.split('/').pop();
        if (route != logoutRoute && route != loginRoute) {
            var token = getLocalVar('tokenLogin');
            var hash = localStorage.getItem('LOGIN_HASH');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if (window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if (eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if (hash) {
                requestConfig.headers['OAuth-Hash'] = hash;
                requestConfig.headers['OAuth-Project'] = window.tekLoginProjectId;
            }
            if (route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

};

Configuration(function(ContextRegister) {
    ContextRegister.register('EngineService', EngineService);
});
